library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity segTest is
    Port (	selector : out  STD_LOGIC_VECTOR (3 downto 0);
				digit : out  STD_LOGIC_VECTOR (6 downto 0);
				BTN0,BTN1,BTN2,BTN3 : in STD_LOGIC;
				clk : in STD_LOGIC);
end segTest;

architecture Behavioral of segTest is
type stateType is (noop, seg1, seg2, seg3, seg4);

signal ctr : integer range 0 to 8192; -- counter for time multiplexed segments and keypresses
signal number : integer range 0 to 10999; -- highest value, 9999+1000 (then reset to 0)
shared variable iVal0, iVal1, iVal2, iVal3 : integer range 0 to 9; -- values for the 4 digits
signal state : stateType;
signal stateDisplay : stateType;
signal startCalculation : STD_LOGIC;

begin

	CALCULATEDIGITS: process(clk)
	variable iNumberTmp : integer range 0 to 9999;
	begin
	if rising_edge(clk) then
	
		if (startCalculation='1') then
			iVal0 := 0;
			iVal1 := 0;
			iVal2 := 0;
			iVal3 := 0;
			state <= seg1;
		else
		
		case state is
			when seg1 =>
				iNumberTmp := number;

			-- 1000er Ziffer
				if (iNumberTmp>=1000) then
					for index in 0 to 9 loop
						exit when (iNumberTmp<1000);
						iVal3 := iVal3 + 1;
						iNumberTmp := iNumberTmp - 1000;
					end loop;
				end if;
				
			-- 100er Ziffer
				if (iNumberTmp>=100) then
					for index in 0 to 9 loop
						exit when (iNumberTmp<100);
						iVal2 := iVal2 + 1;
						iNumberTmp := iNumberTmp - 100;
					end loop;
				end if;
				
			-- 10er Ziffer
				if (iNumberTmp>=10) then
					for index in 0 to 9 loop
						exit when (iNumberTmp<10);
						iVal1 := iVal1 + 1;
						iNumberTmp := iNumberTmp - 10;
					end loop;
				end if;

			-- 1er Ziffer
				iVal0 := iNumberTmp;
				state <= noop;	
			
			when others	 =>
				null;
		end case;

		end if;	
	end if;	
	end process;


	SETDIGITS:process(clk)

	procedure setDigit(number : integer range 0 to 9) is
		begin
			case number is
				when 0 => digit <= "1000000";
				when 1 => digit <= "1111001";
				when 2 => digit <= "0100100";
				when 3 => digit <= "0110000";
				when 4 => digit <= "0011001";
				when 5 => digit <= "0010010";
				when 6 => digit <= "0000010";
				when 7 => digit <= "1111000";
				when 8 => digit <= "0000000";
				when 9 => digit <= "0010000";
				when others =>	digit <= "0111111"; -- display a "-"
			end case;
	end; -- end procedure setDigit

	begin
		if rising_edge(clk) then
			ctr <= ctr + 1;
		
			if (ctr = 8192) then
				case stateDisplay is
					when seg1 =>
						selector <= "1110";
						setDigit(iVal0);
						stateDisplay <= seg2;
					when seg2 =>
						selector <= "1101";
						setDigit(iVal1);
						stateDisplay <= seg3;
					when seg3 =>
						selector <= "1011";
						setDigit(iVal2);
						stateDisplay <= seg4;
					when seg4 =>
						selector <= "0111";
						setDigit(iVal3);
						stateDisplay <= seg1;
					when others =>
						stateDisplay <= seg1;
				end case;
				ctr <= 0;
			end if;

		end if;
		
	end process;



	SETNUMBER:process(clk)
	variable BTN0old, BTN1old, BTN2old, BTN3old : STD_LOGIC;
	variable numberLocal : integer range 0 to 10999;
	begin
		if rising_edge(clk) then
			if (ctr = 8192) then
				
				startCalculation<='1';
				
				if BTN0='1' and BTN0old='0' then
					numberLocal := number + 1;
				elsif BTN1='1' and BTN1old='0' then
					numberLocal := number + 10;
				elsif BTN2='1' and BTN2old='0' then
					numberLocal := number + 100;
				elsif BTN3='1' and BTN3old='0' then
					numberLocal := number + 1000;
				else
					startCalculation<='0';
				end if;
				
				if numberLocal>9999 then
					number<=0;
				else
					number<=numberLocal;
				end if;
			
				BTN0old := BTN0;
				BTN1old := BTN1;
				BTN2old := BTN2;
				BTN3old := BTN3;

			end if;
			
		end if;
	end process;
	
end Behavioral;

